<?php
//////////////////////////////////////////////////////////////////////////
//
//					LitePHP function(Lite框架的其他函数的支持模块)
//					LitePHP 是一个轻量的框架
//					他追求的是速度,以及最方便的.
//					作者:Kelezyb
//					日期:2008/3/15 23:28
//
//////////////////////////////////////////////////////////////////////////


/**
 * 输出变量的内容，通常用于调试
 *
 * @package Core
 *
 * @param mixed $vars 要输出的变量
 * @param string $label
 * @param boolean $return
 */
function dump($vars, $label = '', $return = false) {
    if (ini_get('html_errors')) {
        $content = "<pre>\n";
        if ($label != '') {
            $content .= "<strong>{$label} :</strong>\n";
        }
        $content .= htmlspecialchars(print_r($vars, true));
        $content .= "\n</pre>\n";
    } else {
        $content = $label . " :\n" . print_r($vars, true);
    }
    if ($return) { return $content; }
    echo $content;
    return null;
}	//{{

/**
 * 把数据写入缓存
 * 
 * @param string $key
 * @param mixed $value
 * 
 * @return bool
 * 	 
 */
function writeCache($key, &$data){
	$cacheDir = '_cache';
	if (!is_dir($cacheDir)) {
		mkdir($cacheDir, 0777);
	}
	
	$cacheFile = $cacheDir . DS . md5($key) . '.php';
	
	$data = serialize($data);
	$hash = sprintf('% 32d', crc32($data));
	$prefix = '<?php die(); ?> ';
	
	$data = $prefix . $hash . $data;
	if (!safe_file_put_contents($cacheFile, $data)) {
		return false;
	} else {
		return true;
	}
}

/**
 * 从缓存中读取数据
 * 
 * @param string $key
 * @param int $timer 过期的时间(ms)
 * @param bool $timeIsLifetime 缓存的内容是否过期
 *
 * @return mixed
 * 
 */
function readCache($key, $time = 900, $timeIsLifetime = true) {
	$cacheFile = '_cache' . DS . md5($key) . '.php';
	
	if (!file_exists($cacheFile)) { return false; }
	
	if ($timeIsLifetime && $time == -1) {
		$data = safe_file_get_contents($cacheFile);
		
		$hash = substr($data, 16, 32);
		$data = substr($data, 48);
		if (crc32($data) != $hash || strlen($hash) != 32) {
			return false;
		}
		return $data !== false ? unserialize($data) : false;
	}
	
	$filetime = filemtime($cacheFile);
	if ($timeIsLifetime) {
		if (time() >= $filetime + $time) { return false; }
	} else {
		if ($time >= $filetime) { return false; }
	}
	$data = safe_file_get_contents($cacheFile);
	$hash = substr($data, 16, 32);
	$data = substr($data, 48);
	if (crc32($data) != $hash || strlen($hash) != 32) {
		return false;
	}
	return $data !== false ? unserialize($data) : false;
}

/**
 * 删除缓存
 * 
 * @param string $Key
 */
function purgeCache($key) {
	$cacheFile = '_cache' . DS . md5($key) . '.php';
	
	if (file_exists($cacheFile)) {
		return unlink($cacheFile);
	}
	else{
		return true;
	}
}

/**
 * 获得程序运行的时间()
 * 
 */
function runTimers() {
	global $___lite_loaded_time;
	$result = sprintf('%2.6f', microtime() - $___lite_loaded_time);
	$___lite_loaded_time = microtime();
	return $result;
}

/**
 * 一次性完成打开文件，写入内容，关闭文件三项工作，并且确保写入时不会造成并发冲突
 * 
 * @param string $filename
 * @param string $content
 *
 * @return boolean
 */
function safe_file_put_contents($filename, & $content)
{
    $fp = fopen($filename, 'wb');
    if ($fp) {
        flock($fp, LOCK_EX);
        fwrite($fp, $content);
        flock($fp, LOCK_UN);
        fclose($fp);
        return true;
    } else {
        return false;
    }
}

/**
 * 用共享锁模式打开文件并读取内容，可以避免在并发写入造成的读取不完整问题
 * 
 * @param string $filename
 *
 * @return mixed
 */
function safe_file_get_contents($filename)
{
    $fp = fopen($filename, 'rb');
    if ($fp) {
        flock($fp, LOCK_SH);
        clearstatcache();
        $filesize = filesize($filename);
        if ($filesize > 0) {
            $data = fread($fp, $filesize);
        } else {
        	$data = false;
        }
        flock($fp, LOCK_UN);
        fclose($fp);
        return $data;
    } else {
        return false;
    }
}

/**
 * 重定向浏览器到指定的 URL
 * 
 * @param string $url 要重定向的 url
 * @param int $delay 等待多少秒以后跳转
 * @param bool $js 指示是否返回用于跳转的 JavaScript 代码
 * @param bool $jsWrapped 指示返回 JavaScript 代码时是否使用 <script> 标签进行包装
 * @param bool $return 指示是否返回生成的 JavaScript 代码
 */
function redirect($url, $delay = 0, $js = false, $jsWrapped = true, $return = false)
{
	$delay = (int)$delay;
	if (!$js) {
		if (headers_sent() || $delay > 0) {
			echo <<<EOT
    <html>
    <head>
    <meta http-equiv="refresh" content="{$delay};URL={$url}" />
    </head>
    </html>
EOT;
			exit;
		} else {
			header("Location: {$url}");
			exit;
		}
	}
	
	$out = '';
	if ($jsWrapped) {
		$out .= '<script language="JavaScript" type="text/javascript">';
	}
	if ($delay > 0) {
		$out .= "window.setTimeout(function () { document.location='{$url}'; }, {$delay});";
	} else {
		$out .= "document.location='{$url}';";
	}
	if ($jsWrapped) {
		$out .= '</script>';
	}
	
	if ($return) {
		return $out;
	}
	
	echo $out;
	exit;
}

?>